/*
 *   Copyright 2011 Kevin Kofler <kevin.kofler@chello.at>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "private/componentinstaller_p.h"

#include <kglobal.h>

#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
#include <QSet>
#include <QDBusInterface>
#include <QDBusPendingCall>
#include <QWidget>
#include <QLatin1String>
#include <QStringList>
#include <QTextStream>
#include <QFile>
#include <QDirIterator>
#include <QRegExp>
#endif

namespace Plasma
{

class ComponentInstallerPrivate
{
    public:
#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
        QSet<QString> alreadyPrompted;
#endif
};

class ComponentInstallerSingleton
{
    public:
        ComponentInstaller self;
};

K_GLOBAL_STATIC(ComponentInstallerSingleton, privateComponentInstallerSelf)

ComponentInstaller *ComponentInstaller::self()
{
    return &privateComponentInstallerSelf->self;
}

ComponentInstaller::ComponentInstaller()
    : d(new ComponentInstallerPrivate)
{
}

ComponentInstaller::~ComponentInstaller()
{
    delete d;
}

void ComponentInstaller::installMissingComponent(const QString &type,
                                                 const QString &name,
                                                 QWidget *parent, bool force)
{
#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
    QString searchString = type + '-' + name;

    if (!force) {
        if (d->alreadyPrompted.contains(searchString)) {
            return;
        }
    }

    d->alreadyPrompted.insert(searchString);

    QDBusInterface packageKit(QLatin1String("org.freedesktop.PackageKit"),
                              QLatin1String("/org/freedesktop/PackageKit"),
                              QLatin1String("org.freedesktop.PackageKit.Modify"));
    // We don't check packageKit.isValid() because the service is activated on
    // demand, so it will show up as "not valid".
    WId wid = 0;
#ifndef PLASMA_COMPONENTINSTALLER_NO_QWIDGET
    if (parent) {
        wid = parent->winId();
    }
#else
    Q_UNUSED(parent);
#endif
    QStringList resources;
    resources.append(searchString);
    packageKit.asyncCall(QLatin1String("InstallResources"), (unsigned int) wid,
                         QLatin1String("plasma-service"), resources, QString());
#else
    Q_UNUSED(type);
    Q_UNUSED(name);
    Q_UNUSED(parent);
    Q_UNUSED(force);
#endif
}

QStringList ComponentInstaller::extractDataEngineDependencies(const QString &path,
                                                              const QString &api)
{
    QStringList deps;

#ifdef PLASMA_ENABLE_PACKAGEKIT_SUPPORT
    QStringList nameFilters;
    QRegExp searchRegExp("dataEngine *\\( *\"([^\"]+)\" *\\)");
    if (api.isEmpty()) {
        // no script engine API, this is native C++ code
        nameFilters.append("*.cpp");
        nameFilters.append("*.cxx");
        nameFilters.append("*.cc");
        nameFilters.append("*.C");
        nameFilters.append("*.h");
        nameFilters.append("*.hpp");
        nameFilters.append("*.hxx");
        nameFilters.append("*.hh");
        nameFilters.append("*.H");
    } else if (api == "declarativeappletscript") {
        nameFilters.append("*.qml");
        searchRegExp = QRegExp("(?:^\\s*engine:\\s*|dataEngine *\\( *)\"([^\"]+)\"");
    } else if (api == "javascript") {
        nameFilters.append("*.js");
    } else if (api == "python") {
        nameFilters.append("*.py");
        searchRegExp = QRegExp("dataEngine *\\( *[\'\"]([^\'\"]+)[\'\"] *\\)");
    } else if (api == "ruby-script") {
        nameFilters.append("*.rb");
        searchRegExp = QRegExp("dataEngine *\\( *[\'\"]([^\'\"]+)[\'\"] *\\)");
    } else {
        // dependency extraction not supported for this API
        return deps;
    }

    QDirIterator it(path, nameFilters, QDir::Files | QDir::CaseSensitive,
                    QDirIterator::Subdirectories
                      | QDirIterator::FollowSymlinks);
    while (it.hasNext()) {
        QFile file(it.next());
        file.open(QIODevice::ReadOnly | QIODevice::Text);
        QTextStream stream(&file);
        QString line;
        while (!(line = stream.readLine()).isNull()) {
             int column = 0;
             while ((column = searchRegExp.indexIn(line, column)) != -1) {
                 QString dep = searchRegExp.cap(1);
                 if (!deps.contains(dep)) {
                     deps.append(dep);
                 }
                 column += searchRegExp.matchedLength();
             }
        }
        file.close();
    }
#else
    Q_UNUSED(path);
    Q_UNUSED(api);
#endif

    return deps;
}

} // namespace Plasma
